#!/usr/bin/env node

/**
 * @file Build this project.
 */

"use strict";

process.chdir(__dirname + '/..');

var uiContrib = require('apeman-tmpl-contrib-official/lib/ui_contrib'),
    tasking = require('apeman-dev-commons-tasking'),
    path = require('path'),
    coz = require('coz'),
    glob = require('glob'),
    async = require('async');

var pkg = require('../package.json');

var apDependencies = [
    'apeman-ui-contrib-angular-asserting',
    'apeman-ui-contrib-angular-touching',
    'apeman-ui-contrib-angular-timing',
    'apeman-ui-contrib-angular-dialoging'
];

tasking.runTasks([
    function renderProject(callback) {
        uiContrib.render({
            pkg: pkg,
            ignore: [],
            year: new Date().getFullYear(),
            get signature() {
                try {
                    return require('../.signature');
                } catch (e) {
                    return null;
                }
            },
            get dependencies() {
                var dependencies = {
                    'angular': '~1.4.3',
                    'jquery': '~2.1.4',
                    'hammerjs': '~2.0.4'
                };
                apDependencies.forEach(function (name) {
                    dependencies[name] = pkg["dependencies"][name];
                });
                return dependencies;
            },
            get docs() {
                var docs = {};
                glob.sync('assets/docs/**/*.md').filter(function (filename) {
                    return path.basename(filename) !== 'index.md';
                }).forEach(function (filename) {
                    docs[path.basename(filename, '.md')] = filename;
                });
                return docs;
            }
        }, callback);
    },
    function renderAssets(callback) {
        coz.render([
            'assets/**/.*.bud'
        ], callback);
    },
    function buildDependencies(callback) {
        async.eachSeries(apDependencies, function (name, callback) {
            var dist = require(name);
            async.eachSeries([
                'assets/test/dependencies',
                'docs/demo/resources'
            ], function (dest, callback) {
                dist()({dest: dest}, callback);
            }, callback);
        }, callback);
    },
    function generateThis(callback) {
        var dist = require('../dist');
        async.eachSeries([
            'assets/dist',
            'docs/demo/resources'
        ], function (dest, callback) {
            dist()({dest: dest}, callback);
        }, callback);
    }
], true);
