(function (ng) {
    /**
     * @ngdoc module
     * @module apeman-ui-contrib-angular-timing
     * @name apeman-ui-contrib-angular-timing
     * @description apeman ui module for angular to work with timers.
     */

    "use strict";

    ng
        .module('apeman-ui-contrib-angular-timing', [
        ]);
})(angular);(function (ng) {

    /**
     * @ngdoc type
     * @module apeman-ui-contrib-angular-timing
     * @name ApTimer
     * @description A timer context.
     */

    "use strict";

    ng
        .module('apeman-ui-contrib-angular-timing')
        .factory('ApTimer', function ($timeout) {
            function ApTimer() {
                var s = this;
                s._timers = {};
            }

            ApTimer.prototype = {
                /**
                 * @ngdoc method
                 * @name ApTimer#cancel
                 * @description Cancel a task.
                 * @param {string} reserveId - Id of reservation to cancel.
                 * @returns {ApTimer} - Returns self.
                 */
                cancel: function (reserveId) {
                    var s = this,
                        timers = s._timers;
                    var timer = timers[reserveId];
                    $timeout.cancel(timer);
                    return s;
                },
                /**
                 * @ngdoc method
                 * @name ApTimer#create
                 * @module apeman-ui-contrib-angular-timing
                 * @description Cancel all tasks.
                 * @returns {ApTimer} - Returns self.
                 */
                cancelAll: function () {
                    var s = this,
                        timers = s._timers;
                    Object.keys(timers).forEach(function (reserveId) {
                        var timer = timers[reserveId];
                        $timeout.cancel(timer);
                    });
                    return s;
                },
                /**
                 * @ngdoc method
                 * @name ApTimer#reserve
                 * @description Reserve a task.
                 * @param {function} task - Task to run.
                 * @param {number} delay - When to exec.
                 * @returns {number} - Id for reservation.
                 */
                reserve: function (task, delay) {
                    var s = this,
                        reserveId = s._now() + delay;
                    s._timers[reserveId] = $timeout(function () {
                        task();
                        delete s._timers[reserveId];
                    }, delay || 0);
                    return reserveId;
                },
                _timers: null,
                _now: function () {
                    return new Date().getTime();
                }
            };
            return ApTimer;
        });
})(angular);(function (ng) {
    /**
     * @ngdoc service
     * @module apeman-ui-contrib-angular-timing
     * @name apTime
     * @description Reserve and cancel tasks using $timeout.
     * This is an instance of apeman-ui-contrib-angular-timing.ApTimer
     */

    "use strict";

    ng
        .module('apeman-ui-contrib-angular-timing')
        .service('apTime', function ApTime(ApTimer) {
            var s = this;
            s.__proto__ = new ApTimer();

            s.ApTimer = ApTimer;

            /**
             * @ngdoc method
             * @name apTime#cancel
             * @description Cancel a task.
             * @param {string} reserveId - Id of reservation to cancel.
             * @returns {apTime} - Returns self.
             */
            /**
             * @ngdoc method
             * @name apTime#create
             * @module apeman-ui-contrib-angular-timing
             * @description Cancel all tasks.
             * @returns {apTime} - Returns self.
             */
            /**
             * @ngdoc method
             * @name apTime#reserve
             * @description Reserve a task.
             * @param {function} task - Task to run.
             * @param {number} delay - When to exec.
             * @returns {number} - Id for reservation.
             */

            /**
             * @ngdoc method
             * @name apTime#create
             * @module apeman-ui-contrib-angular-timing
             * @description Create a new context.
             */
            s.create = function () {
                return new ApTimer();
            };

        });

})(angular);