(function (ng) {
    /**
     * @ngdoc module
     * @module apeman-ui-contrib-angular-touching
     * @name apeman-ui-contrib-angular-touching
     * @description
     * apeman ui module for angular to handle touches.
     */

    "use strict";

    ng
        .module('apeman-ui-contrib-angular-touching', [
            'apeman-ui-contrib-angular-timing'
        ]);
})(angular);
(function (ng, $, document) {

    /**
     * @ngdoc directive
     * @module apeman-ui-contrib-angular-touching
     * @name apTap
     * @description Detect tap event.
     * @restrict A
     */

    "use strict";

    var SCROLL_LOCK_DURATION = 100,
        TAP_LOCK_DURATION = 200;

    ng
        .module('apeman-ui-contrib-angular-touching')
        .directive('apTap', function apTap(apTime) {

            return {
                restrict: 'A',
                link: function (scope, elm, attr) {
                    var $elm = $(elm).addClass('ap-tap'),
                        $document = $(document);

                    var timer = apTime.create(),
                        lastTapDate = null,
                        lastScrollDate = null;

                    function _lockByScroll(now) {
                        return lastScrollDate && (now - lastScrollDate < SCROLL_LOCK_DURATION);
                    }

                    function _lockByTap(now) {
                        return lastTapDate && (now - lastTapDate < TAP_LOCK_DURATION);
                    }

                    function onclick() {
                        var now = new Date();
                        var locked = _lockByScroll(now) || _lockByTap(now);
                        if (locked) {
                            return;
                        }
                        timer.reserve(function (e) {
                            $elm.trigger('ap:tap:tapped');
                            scope.$eval(attr['apTap'], {
                                $event: e
                            });
                        });
                    }

                    function onscroll() {
                        lastScrollDate = new Date();
                    }

                    $elm.on('click', onclick);
                    $document.on('ap:scroll:scrolled', onscroll);


                    scope.$on('$destroy', function () {
                        timer.cancelAll();
                        $elm.off('click', onclick);
                        $document.off('ap:scroll:scrolled', onscroll);
                    });
                }
            }
        });

})(angular, jQuery, window.document);

(function (ng, $, document) {

    /**
     * @ngdoc directive
     * @module apeman-ui-contrib-angular-touching
     * @name apTouchOutside
     * @description Detect touch element outside.
     * @restrict A
     */

    "use strict";

    ng
        .module('apeman-ui-contrib-angular-touching')
        .directive('apTouchOutside', function apTouchOutside(apTime,
                                                             apTouching) {


            var BLOCK_DURATION = 200; //Timer to block double trigger.
            var watchingEvents = ['mousedown', 'touchstart'];

            return {
                restrict: 'A',
                compile: function () {
                    return {
                        post: function (scope, elm, attr) {
                            var $document = $(document),
                                $elm = $(elm).addClass('ap-touch-outside'),
                                timer = apTime.create(),
                                blocked = false;


                            function touchstart(e) {
                                if (!e.target) {
                                    return e.target;
                                }
                                if (blocked) {
                                    return;
                                }
                                blocked = true;
                                timer.reserve(function () {
                                    blocked = false;
                                }, BLOCK_DURATION);

                                var isOutside = ($elm[0] != e.target) && ($elm.has(e.target).size() == 0);
                                if (isOutside) {
                                    timer.reserve(function () {
                                        $elm.trigger('ap:touch:outside');
                                        scope.$eval(attr['apTouchOutside']);
                                    });
                                }
                            }

                            apTouching.bindEvents($document, watchingEvents, touchstart);
                            scope.$on('$destroy', function () {
                                timer.cancelAll();
                                apTouching.unbindEvents($document, watchingEvents, touchstart);
                            });
                        }
                    }
                }
            }
        });

})(angular, jQuery, window.document);

(function (ng, $, document) {

    /**
     * @ngdoc directive
     * @module apeman-ui-contrib-angular-touching
     * @name apTouchStart
     * @description Detect touch element start.
     * @restrict A
     */

    "use strict";

    ng
        .module('apeman-ui-contrib-angular-touching')
        .directive('apTouchStart', function apTouchOutside(apTime,
                                                           apTouching) {

            var BLOCK_DURATION = 200; //Timer to block double trigger.

            var watchingEvents = ['mousedown', 'touchstart'];
            return {
                restrict: 'A',
                compile: function () {
                    return {
                        post: function (scope, elm, attr) {
                            var $elm = $(elm).addClass('ap-touch-start'),
                                timer = apTime.create(),
                                blocked = false;

                            function touchstart(e) {
                                if (blocked) {
                                    return;
                                }
                                blocked = true;
                                timer.reserve(function () {
                                    blocked = false;
                                }, BLOCK_DURATION);

                                timer.reserve(function () {
                                    $elm.trigger('ap:touch:start');
                                    scope.$eval(attr['apTouchStart']);
                                });
                            }

                            apTouching.bindEvents($elm, watchingEvents, touchstart);
                            scope.$on('$destroy', function () {
                                timer.cancelAll();
                                apTouching.unbindEvents($elm, watchingEvents, touchstart);
                            });
                        }
                    }
                }
            }
        });

})(angular, jQuery, window.document);

(function (ng, $, Hammer) {
    /**
     * @ngdoc service
     * @name apHammer
     * @module apeman-ui-contrib-angular-touching
     */

    "use strict";

    ng
        .module('apeman-ui-contrib-angular-touching')
        .service('apHammer', function apHammer() {
            var s = this;
            /**
             * @ngdoc method
             * @name apHammer#panHammer
             * @description Hammer for all direction pn.
             * @param {HTMLElement} elm - Element to work with.
             * @returns {Hammer} - A new hammer.
             */
            s.panHammer = function panHammer(elm) {
                var hammer = new Hammer($(elm).get(0));
                hammer.get('pan').set({
                    direction: Hammer.DIRECTION_ALL
                });
                return hammer;
            };
            /**
             * @ngdoc method
             * @name apHammer#addListners
             * @description Add multiple listeners.
             * @param {Hammer} hammer - Hammer instance to work with.
             * @param {object} listeners - Listeners to add.
             */
            s.addListeners = function addListeners(hammer, listeners) {
                Object.keys(listeners || {}).forEach(function (event) {
                    var listener = listeners[event];
                    hammer.on(event, listener);
                });
            };
            /**
             * @ngdoc method
             * @name apHammer#removeListners
             * @description Remove multiple listeners.
             * @param {Hammer} hammer - Hammer instance to work with.
             * @param {object} listeners - Listeners to remove.
             */
            s.removeListeners = function removeListeners(hammer, listeners) {
                Object.keys(listeners || {}).forEach(function (event) {
                    var listener = listeners[event];
                    hammer.off(event, listener);
                });
            };
        });


})(angular, jQuery, Hammer);
(function (ng, window) {
    /**
     * @ngdoc service
     * @name apTouching
     * @description Touch Utility
     * @module apeman-ui-contrib-angular-touching
     */
    "use strict";

    ng
        .module('apeman-ui-contrib-angular-touching')
        .service('apTouching', function apTouching() {
            var s = this;
            /**
             * @ngdoc method
             * @name apTouching#isTouchable
             * @description Detect touchable or not.
             * @returns {boolean} - Is touchable or not.
             */
            s.isTouchable = function () {
                return !!('ontouchstart' in window || 'onmsgesturechange' in window);
            };


            s.bindEvents = function ($elm, events, handler) {
                for (var i = 0; i < events.length; i++) {
                    $elm.on(events[i], handler);
                }
            };
            s.unbindEvents = function ($elm, events, handler) {
                for (var i = 0; i < events.length; i++) {
                    $elm.off(events[i], handler);
                }
            };


        });
})(angular, window);
